from eoplatform.console import console


# from eoplatform.platforms.Landsat8.downloader import Downloader as L8Downloader


def download(name: str) -> None:
    console.print(
        "[red bold encircle]:x:  downloading not yet implemented. Stay tuned!"
    )
    return
